// Matrix.java

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.applet.Applet;
import java.io.InputStream;         // Lets you read files
import java.io.StreamTokenizer;     // Lets you interprit what you read
import java.net.*;                  // Let you open a URL
import java.io.IOException;         // Deals with io exceptions
import java.io.*;

//import javax.swing.*;

public class Laplace extends Applet {
  public void init() {

    variableSetting();

    Panel main = new Panel(new BorderLayout());
    //main.setBackground(colorBackground);

    if ( inputName == null ) {
      laplaceMain = new Matrix(nProcCol, nProcRow, matrixCol, matrixRow, this, colorBackground, colorBoundary, colorBuffer, colorLocalOld, colorLocalNew, colorLocalNewChanged);
    }
    else {
      laplaceMain = new Matrix(nProcCol, nProcRow, matrixCol, matrixRow, inputName, documentBase, this, colorBackground, colorBoundary, colorBuffer, colorLocalOld, colorLocalNew, colorLocalNewChanged);
    }

    // the main part of the applet
    main.add("Center", laplaceMain);

    // Agenda
    OutlinePanel outBox = new OutlinePanel(1);
    Agenda agenda = new Agenda(colorBackground, colorBoundary, colorBuffer, colorLocalOld, colorLocalNew, colorLocalNewChanged);
    outBox.add("Center", agenda);

    main.add("North", outBox);

    // control part
    startButton = new Button("START");
    startButton.setFont(new Font("Courier", Font.BOLD, 14));
    startButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	laplaceMain.startApplet();
	startButton.setEnabled(false);
	stopButton.setEnabled(true);
	resumeButton.setEnabled(false);
	nextStepButton.setEnabled(true);
      }
    });

    stopButton = new Button("STOP");
    stopButton.setFont(new Font("Courier", Font.BOLD, 14));
    stopButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	laplaceMain.suspendApplet();
	stopButton.setEnabled(false);
	resumeButton.setEnabled(true);
	nextStepButton.setEnabled(true);
      }
    });

    resumeButton = new Button("RESUME");
    resumeButton.setFont(new Font("Courier", Font.BOLD, 14));
    resumeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	laplaceMain.resumeApplet();
	stopButton.setEnabled(true);
	resumeButton.setEnabled(false);
	nextStepButton.setEnabled(true);
      }
    });
    resumeButton.setEnabled(false);

    nextStepButton = new Button("NEXT STEP");
    nextStepButton.setFont(new Font("Courier", Font.BOLD, 14));
    nextStepButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	laplaceMain.nextStepApplet();
	stopButton.setEnabled(true);
	resumeButton.setEnabled(true);
      }
    });
    nextStepButton.setEnabled(false);

    speedUpButton = new Button("Speed Up");
    speedUpButton.setFont(new Font("Courier", Font.BOLD, 14));
    speedUpButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
      }
    });

    speedDownButton = new Button("Speed Down");
    speedDownButton.setFont(new Font("Courier", Font.BOLD, 14));
    speedDownButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
      }
    });

    demoSpeedControl = new Scroller(Scrollbar.HORIZONTAL);
    demoSpeedControl.setValue((int)Math.round((Math.log(scale)/Math.log(2.0)*step)), 1, -step*maxp, step*maxp+1, false);

    /**
    demoSpeedControl.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	double v = getScale();
	demoSpeedLabel.setText(String.valueOf((int)(v*100+0.5))+"%");
	//Resize.setActive(true);
      }
    });
    **/

    //demoSpeedLabel = new Label3D(String.valueOf((int)(scale*100+0.5)), 0);
    laplaceMain.setDemoSpeed((int)(scale*100+0.5));

    Label tmpLabel = new Label("Simulation Speed");

    /**
    Panel p = new Panel();
    p.setLayout(new GridLayout(1,5));
    p.add(startButton);
    p.add(stopButton);
    p.add(resumeButton);
    p.add(demoSpeedControl);
    p.add(demoSpeedLabel);
    **/

    Panel p = new Panel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints gridControl = new GridBagConstraints();
    p.setLayout(gridbag);

    gridControl.insets = new Insets(0,10,0,10);
    gridControl.gridwidth = 2;

    gridControl.gridx = 0;
    gridControl.gridy = 0;
    gridbag.setConstraints(startButton, gridControl);
    p.add(startButton);

    gridControl.gridx = 2;
    gridControl.gridy = 0;
    gridbag.setConstraints(stopButton, gridControl);
    p.add(stopButton);

    gridControl.gridx = 4;
    gridControl.gridy = 0;
    gridbag.setConstraints(resumeButton, gridControl);
    p.add(resumeButton);

    gridControl.gridx = 6;
    gridControl.gridy = 0;
    gridbag.setConstraints(nextStepButton, gridControl);
    p.add(nextStepButton);

    gridControl.insets = new Insets(0,10,0,0);
    gridControl.gridwidth = 4;
    gridControl.gridx = 8;
    gridControl.gridy = 0;
    gridbag.setConstraints(tmpLabel, gridControl);
    p.add(tmpLabel);

    gridControl.insets = new Insets(0,0,0,10);
    gridControl.gridwidth = 6;
    gridControl.gridx = 12;
    gridControl.gridy = 0;
    gridbag.setConstraints(demoSpeedControl, gridControl);
    p.add(demoSpeedControl);

    /**
    p.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	printLine("panel action");
      }
    });
    **/

    //p.add(speedUpButton);
    //p.add(speedDownButton);

    main.add("South", p);

    add(main);
  }

  public double getScale() {
    return Math.pow(2.0, ((double)demoSpeedControl.getValue())/step);
  }

  public boolean action(Event ev, Object arg) {
    if ( arg == demoSpeedControl ) {
      double v = getScale();
      //demoSpeedLabel.setText(String.valueOf((int)(v*100+0.5)));
      laplaceMain.setDemoSpeed((int)(v*100+0.5));
      return true;
    }
    return true;
  }

  public void variableSetting() {

    try {
      nProcCol  = (Integer.valueOf(getParameter("numberOfProcessorsCol"))).intValue();
      nProcRow  = (Integer.valueOf(getParameter("numberOfProcessorsRow"))).intValue();

      matrixCol = (Integer.valueOf(getParameter("matrixSizeCol"))).intValue();
      matrixRow = (Integer.valueOf(getParameter("matrixSizeRow"))).intValue();
    }
    catch(Exception e) {
      nProcCol  = 4;
      nProcRow  = 4;
      matrixCol = 12;
      matrixRow = 12;
    }

    try {
      inputName = getParameter("inputfile");
    } 
    catch(Exception e) {
      inputName = null;
    }
    documentBase = getDocumentBase();

    // Color setting
    colorBackground = Color.gray;
    colorBoundary   = Color.yellow;
    colorBuffer     = Color.cyan;
    colorLocalOld   = Color.pink;
    colorLocalNew   = Color.white;
    colorLocalNewChanged = Color.magenta;
  }

  public void printLine(String printData) {
    if ( debugApplet ) {
      System.out.println("<Laplace> " + printData);
    }
  }

  protected int nProcCol = 0;
  protected int nProcRow = 0;
  protected int matrixCol = 0;
  protected int matrixRow = 0;
  protected String inputName = null;
  protected URL documentBase = null;

  protected int numberOfProcessors = 0;
  protected int mSquares = 0;
  protected int nSquares = 0;

  protected Matrix laplaceMain;

  protected Color colorBackground;
  protected Color colorBoundary;
  protected Color colorBuffer;
  protected Color colorLocalOld;
  protected Color colorLocalNew;
  protected Color colorLocalNewChanged;

  protected Vector processors = new Vector();
  protected double matrixData[][];

  protected Button startButton;
  protected Button stopButton;
  protected Button resumeButton;
  protected Button nextStepButton;
  protected Button speedUpButton;
  protected Button speedDownButton;
  protected Scroller demoSpeedControl;

  protected Label3D demoSpeedLabel;
  protected double scale = 1.0;
  protected int maxp = 2;
  protected int step = 50;

  protected Image upperArrow;
  protected Image downArrow;
  protected Image rightArrow;
  protected Image leftArrow;
  protected Image blankArrow;
  protected boolean debugApplet = false;
}
