// Matrix.java

import java.awt.*;
import java.util.Vector;
import java.applet.Applet;
import java.io.InputStream;         // Lets you read files
import java.io.StreamTokenizer;     // Lets you interprit what you read
import java.net.*;                  // Let you open a URL
import java.io.IOException;         // Deals with io exceptions
import java.io.*;

//import javax.swing.*;

public class Matrix extends java.awt.Panel {
  public Matrix() {
    setupGUI();
  }

  public Matrix(int _nProcCol, int _nProcRow, int _matrixCol, int _matrixRow, Laplace _parent, Color _colorBackground, Color _colorBoundary, Color _colorBuffer, Color _colorLocalOld, Color _colorLocalNew, Color _colorLocalNewChanged) {
    nProcCol = _nProcCol;
    nProcRow = _nProcRow;
    matrixCol = _matrixCol;
    matrixRow = _matrixRow;
    parent = _parent;

    colorBackground = _colorBackground;
    colorBoundary   = _colorBoundary;
    colorBuffer     = _colorBuffer;
    colorLocalOld   = _colorLocalOld;
    colorLocalNew   = _colorLocalNew;
    colorLocalNewChanged = _colorLocalNewChanged;

    setupGUI();
  }

  public Matrix(int _nProcCol, int _nProcRow, int _matrixCol, int _matrixRow, String _inputfile, URL _documentBase, Laplace _parent, Color _colorBackground, Color _colorBoundary, Color _colorBuffer, Color _colorLocalOld, Color _colorLocalNew, Color _colorLocalNewChanged) {
    nProcCol = _nProcCol;
    nProcRow = _nProcRow;
    matrixCol = _matrixCol;
    matrixRow = _matrixRow;

    inputfile = _inputfile;
    documentBase = _documentBase;
    parent = _parent;

    colorBackground = _colorBackground;
    colorBoundary   = _colorBoundary;
    colorBuffer     = _colorBuffer;
    colorLocalOld   = _colorLocalOld;
    colorLocalNew   = _colorLocalNew;
    colorLocalNewChanged = _colorLocalNewChanged;

    setupGUI();
  }

  protected void setupGUI() {

    // sample values
    variableSetting();
    matrixData = new double[matrixCol][matrixRow];
    if ( inputfile == null ) {
      int k = 0;
      for ( int i = 0 ; i < matrixRow ; i++ ) {
	for ( int j = 0 ; j < matrixCol ; j++ ) {
	  /**/
	  if ( i == 0 || 
	       j == 0 || 
	       i == matrixRow - 1 || 
	       j == matrixCol - 1 ) {
	    matrixData[j][i] = 1.;
	  }
	  else {
	    matrixData[j][i] = 0.;
	  }
	  /**/
	  //matrixData[j][i] = k++;
	}
      }
    }
    else {
      readInitialData();
    }

    // print out the input data
    printLine("Input Data");
    for ( int i = 0 ; i < matrixRow ; i++ ) {
      String tmpRow = i + ":  ";
      for ( int j = 0 ; j < matrixCol ; j++ ) {
	tmpRow = tmpRow + matrixData[j][i] + "  ";
      }
      printLine(tmpRow);
    }

    syncProcessors = new int[numberOfProcessors];
    for ( int i = 0 ; i < numberOfProcessors ; i++ ) {
      syncProcessors[i] = -1;
    }

    // create animation class for each processor
    Panel main = new Panel(new GridLayout(nProcRow, nProcCol, 0, 0));
    main.setBackground(colorBackground);

    AnimationPanel processMain;
    for(int i = 0; i < numberOfProcessors; i++) {
      syncProcessors[i] = -1;
      processMain = new AnimationPanel(i, nProcCol, nProcRow, matrixCol, matrixRow, colorBackground, colorBoundary, colorBuffer, colorLocalOld, colorLocalNew, colorLocalNewChanged, this, parent);
      processors.addElement(processMain);
      main.add(processMain);
    }
    add(main);

  }


  public void updateSyncInfo(int processorID, int step) {
    boolean syncDone = true;

    syncProcessors[processorID] = step;

    //printLine("sync " + processorID + "  step " + step);

    for(int i = 0; i < numberOfProcessors; i++) {
      if ( syncProcessors[i] != step ) {
	syncDone = false;
      }
    }

    if ( !syncDone ) {
      return;
    }

    printLine("sync " + processorID + "  step " + step + "   sync done");

    for(int i = 0; i < processors.size() ; i++) {
      ((AnimationPanel)processors.elementAt(i)).processMain.processor.syncDemonstration(step);
    }
  }

  public void variableSetting() {

    numberOfProcessors = nProcCol * nProcRow;
    mSquares = matrixCol / nProcCol;
    nSquares = matrixRow / nProcRow;

  }

  public void readInitialData() {

    try {
      InputStream instream = new URL(documentBase, inputfile).openStream();

      Reader reader = new BufferedReader(new InputStreamReader(instream));
      StreamTokenizer intokens = new StreamTokenizer(reader);

      for ( int i = 0 ; i < matrixRow ; i++ ) {
	for ( int j = 0 ; j < matrixCol ; j++ ) {
	  intokens.nextToken();
	  matrixData[j][i] = (double) intokens.nval;
	}
      }

    }
    catch (Exception e) {
      int k = 0;
      for ( int i = 0 ; i < matrixRow ; i++ ) {
	for ( int j = 0 ; j < matrixCol ; j++ ) {
	  matrixData[j][i] = k++;
	  /**
	  if ( i == 0 || 
	       j == 0 || 
	       i == matrixRow + 1 || 
	       j == matrixCol + 1 ) {
	    matrixData[j][i] = 1.;
	  }
	  else {
	    matrixData[j][i] = 0.;
	  }
	  **/
	}
      }
    }

    /**/
    // print out the input data
    printLine("Input Data");
    for ( int i = 0 ; i < matrixRow ; i++ ) {
      String tmpRow = i + ":  ";
      for ( int j = 0 ; j < matrixCol ; j++ ) {
	tmpRow = tmpRow + matrixData[j][i] + "  ";
      }
      printLine(tmpRow);
    }

    double tmpData[][];
    tmpData = new double[matrixCol][matrixRow];
    tmpData = getMatrix();
    printLine("Input Data");
    for ( int i = 0 ; i < matrixRow ; i++ ) {
      String tmpRow = i + ":  ";
      for ( int j = 0 ; j < matrixCol ; j++ ) {
	tmpRow = tmpRow + tmpData[j][i] + "  ";
      }
      printLine(tmpRow);
    }
    /**/

    double aa = 12.3456;
    int tmpInt = (int)(aa * 100);
    printLine("test " + tmpInt);

    int tmpInt2 = (int)(12 / 5);
    int tmpInt3 = (int)(12 % 5);

    printLine("test2 " + tmpInt2 + " " + tmpInt3);

  }

  public Vector getProcessors() {
    return processors;
  }

  public double[][] getMatrix() {
    return matrixData;
  }

  public void startApplet() {
    for ( int i = 0 ; i < processors.size() ; i++ ) {
      //((PartitionPanel)processors.elementAt(i)).startProcessor();
      ((AnimationPanel)processors.elementAt(i)).startProcessor();
    }
  }

  public void suspendApplet() {
    for ( int i = 0 ; i < processors.size() ; i++ ) {
      //((PartitionPanel)processors.elementAt(i)).suspendProcessor();
      ((AnimationPanel)processors.elementAt(i)).suspendProcessor();
    }
  }

  public void resumeApplet() {
    for ( int i = 0 ; i < processors.size() ; i++ ) {
      //((PartitionPanel)processors.elementAt(i)).resumeProcessor();
      ((AnimationPanel)processors.elementAt(i)).resumeProcessor();
    }
  }

  public void nextStepApplet() {
    for ( int i = 0 ; i < processors.size() ; i++ ) {
      //((PartitionPanel)processors.elementAt(i)).resumeProcessor();
      ((AnimationPanel)processors.elementAt(i)).nextStepProcessor();
    }
  }

  public void setDemoSpeed(int speed) {
    for ( int i = 0 ; i < processors.size() ; i++ ) {
      //((PartitionPanel)processors.elementAt(i)).setDemoSpeed(speed);
      ((AnimationPanel)processors.elementAt(i)).setDemoSpeed(speed);
    }
  }

  public void printLine(String printData) {
    if ( debugApplet ) {
      System.out.println("<Laplace> " + printData);
    }
  }

  protected int nProcCol = 0;
  protected int nProcRow = 0;
  protected int matrixCol = 0;
  protected int matrixRow = 0;
  protected String inputfile = null;
  protected URL documentBase = null;
  protected Laplace parent;

  protected int numberOfProcessors = 0;
  protected int mSquares = 0;
  protected int nSquares = 0;

  protected Color colorBackground;
  protected Color colorBoundary;
  protected Color colorBuffer;
  protected Color colorLocalOld;
  protected Color colorLocalNew;
  protected Color colorLocalNewChanged;

  protected Vector processors = new Vector();
  protected double matrixData[][];

  protected boolean debugApplet = false;

  protected int syncProcessors[];
}
