import java.util.Vector;
import java.awt.*;

public class ProcessorPanel extends java.awt.Panel implements java.lang.Runnable {
  public ProcessorPanel(int m, int n) {
    localRow = m;
    localCol = n;	

    setupGUI();

    processorThread = new Thread(this);
  }

  public ProcessorPanel(int m, int n, Color _colorLocalOld, Color _colorLocalNew, Color _colorLocalNewChanged, Color _colorBoundary, Color _colorBackground, int _startRowIndex, int _endRowIndex, int _startColIndex, int _endColIndex, boolean _n, boolean _s, boolean _e, boolean _w, PartitionPanel _parent) {
    localRow = m;
    localCol = n;	

    colorLocalOld   = _colorLocalOld;
    colorLocalNew   = _colorLocalNew;
    colorLocalNewChanged = _colorLocalNewChanged;
    colorBoundary = _colorBoundary;
    colorBackground = _colorBackground;

    startRowIndex = _startRowIndex;
    startColIndex = _startColIndex;
    endRowIndex = _endRowIndex;
    endColIndex = _endColIndex;

    northNeighbor = _n;
    southNeighbor = _s;
    eastNeighbor = _e;
    westNeighbor = _w;

    parent = _parent;

    setupGUI();

    processorThread = new Thread(this);
  }

  public void setRowsPerPartition(int rows) {
    localRow = rows;      
    redoGUI();
  }

  public int getRowsPerPartition() {
    return localRow;        
  }

  public void setColsPerPartition(int columns) {
    localCol = columns;     
    redoGUI();
  }

  public int getColsPerPartition() {
    return localCol;        
  }

  protected void redoGUI() {
    squares.removeAllElements();
    removeAll();
    setupGUI();
  }

  protected void setupGUI() {        
   setLayout(new GridLayout(localRow, localCol, 2, 2));

   // setup the each data in the processor by their location
   for( int i = 0; i < localRow ; i++ ) {
      Vector tmpVector = new Vector();
      tmpVector.removeAllElements();
      for( int j = 0 ; j < localCol ; j++ ) {
	OutlinePanel outBox = new OutlinePanel(1);
	SquarePanel square;
	if ( ( !northNeighbor && i == 0 ) ||
	     ( !southNeighbor && i == localRow - 1 ) ||
	     ( !westNeighbor  && j == 0 ) ||
	     ( !eastNeighbor  && j == localCol - 1 ) ) {

	  if ( ( !northNeighbor && i == 0 ) ||
	       ( !northNeighbor && !westNeighbor && j == 0 && i == 0 ) ||
	       ( !northNeighbor && !eastNeighbor && j == localCol - 1 && i == 0 ) ) {
	    square = new SquarePanel(colorBoundary, colorBackground, false);
	  }
	  else {
	    square = new SquarePanel(colorBoundary, colorBackground, true);
	  }
	}
	else {
	  square = new SquarePanel(colorLocalOld, colorLocalNew);
	}
	add(square);
	tmpVector.addElement(square);
      }
      squares.addElement(tmpVector);
    }
  }

  public void run() {
    // setup the neighbors
    if ( parent.northNeighbor ) {
      northBufferFilled = false;
    }
    else {
      northBufferFilled = true;
    }
    if ( parent.southNeighbor ) {
      southBufferFilled = false;
    }
    else {
      southBufferFilled = true;
    }
    if ( parent.eastNeighbor ) {
      eastBufferFilled  = false;
    }
    else {
      eastBufferFilled = true;
    }
    if ( parent.westNeighbor ) {
      westBufferFilled  = false;
    }
    else {
      westBufferFilled = true;
    }

    syncDemo = false;


    try {
      processorThread.sleep(demoSpeed);
    }
    catch(Exception e) {
    }

    // exchange the data
    sendData();

    try {
      processorThread.sleep(demoSpeed);
    }
    catch(Exception e) {
    }

    // sync point
    if ( !bufferFilled ) {
      try {
	synchronized(this) {
	  while ( !bufferFilled ) {
	    wait();
	  }
	}
      } 
      catch (InterruptedException e){
      }
    }

    receiveData();

    try {
      processorThread.sleep(demoSpeed);
    }
    catch(Exception e) {
    }

    currentStep = 0;
    parent.parent.updateSyncInfo(parent.myID, currentStep);

    // sync point
    if ( !syncDemo ) {
      try {
	synchronized(this) {
	  while ( !syncDemo ) {
	    wait();
	  }
	}
      } 
      catch (InterruptedException e){
      }
    }

    syncDemo = false;
    bufferFilled = false;

    if ( parent.northNeighbor ) {
      northBufferFilled = false;
    }
    else {
      northBufferFilled = true;
    }
    if ( parent.southNeighbor ) {
      southBufferFilled = false;
    }
    else {
      southBufferFilled = true;
    }
    if ( parent.eastNeighbor ) {
      eastBufferFilled  = false;
    }
    else {
	eastBufferFilled = true;
    }
    if ( parent.westNeighbor ) {
      westBufferFilled  = false;
    }
    else {
      westBufferFilled = true;
    }

    parent.returnBufferColor();

    if ( stepDemo ) {
      stepRun = false;
    }
    else {
      stepRun = true;
    }

    // real computation
    for ( int k = 1 ; k < k+1 ; k++ ) {

      currentStep = k * 10 + 1;
      parent.parent.updateSyncInfo(parent.myID, currentStep);

      if ( !syncDemo ) {
	try {
	  synchronized(this) {
	    while ( !syncDemo ) {
	      wait();
	    }
	  }
	} 
	catch (InterruptedException e){
	}
      }

      syncDemo = false;

      // stop point for step simulation
      if ( !stepRun ) {
	try {
	  synchronized(this) {
	    while ( !stepRun ) {
	      wait();
	    }
	  }
	}
	catch (InterruptedException e){
	}
      }

      if ( stepDemo ) {
	stepRun = false;
      }
      else {
	stepRun = true;
      }

      for ( int i = startRowIndex+1 ; i <= endRowIndex ; i++ ) {
	for ( int j = startColIndex+1 ; j <= endColIndex ; j++ ) {
	  parent.newMatrixData[j][i] = 
	    0.25 * ( parent.matrixData[j+1][i] + 
		     parent.matrixData[j-1][i] + 
		     parent.matrixData[j][i-1] + 
		     parent.matrixData[j][i+1] );

	  Double tmpD = new Double(parent.newMatrixData[j][i]);
	  String tmpS = tmpD.toString();
	  //parent.parent.printLine("my ID: " + parent.myID + "   " + tmpS);
	  ((SquarePanel)(((Vector) squares.elementAt(i-1)).elementAt(j-1))).setBottomText(tmpS);
	  ((SquarePanel)(((Vector) squares.elementAt(i-1)).elementAt(j-1))).setBottomBackground(colorLocalNewChanged);
	  try {
	    processorThread.sleep(demoSpeed);
	  }
	  catch(Exception e) {
	  }

	  // suspend & resume point
	  if ( !demoRun ) {
	    try {
	      synchronized(this) {
		while ( !demoRun ) {
		  wait();
		}
	      }
	    }
	    catch (InterruptedException e){
	    }
	  }

	}
      }

      // suspend & resume point
      if ( !demoRun ) {
	try {
	  synchronized(this) {
	    while ( !demoRun ) {
	      wait();
	    }
	  }
	}
	catch (InterruptedException e){
	}
      }

      currentStep = k * 10 + 2;
      parent.parent.updateSyncInfo(parent.myID, currentStep);

      if ( !syncDemo ) {
	try {
	  synchronized(this) {
	    while ( !syncDemo ) {
	      wait();
	    }
	  }
	} 
	catch (InterruptedException e){
	}
      }

      syncDemo = false;

      // stop point for step simulation
      if ( !stepRun ) {
	try {
	  synchronized(this) {
	    while ( !stepRun ) {
	      wait();
	    }
	  }
	}
	catch (InterruptedException e){
	}
      }

      if ( stepDemo ) {
	stepRun = false;
      }
      else {
	stepRun = true;
      }

      sendData();

      try {
	processorThread.sleep(demoSpeed);
      }
      catch(Exception e) {
      }

      if ( !bufferFilled ) {
	try {
	  synchronized(this) {
	    while ( !bufferFilled ) {
	      wait();
	    }
	  }
	} 
	catch (InterruptedException e){
	}
      }

      try {
	processorThread.sleep(demoSpeed);
      }
      catch(Exception e) {
      }

      // suspend & resume point
      if ( !demoRun ) {
	try {
	  synchronized(this) {
	    while ( !demoRun ) {
	      wait();
	    }
	  }
	}
	catch (InterruptedException e){
	}
      }


      receiveData();

      try {
	processorThread.sleep(demoSpeed);
      }
      catch(Exception e) {
      }

      /**
      parent.parent.updateSyncInfo(parent.myID, k);

      if ( !syncDemo ) {
	try {
	  synchronized(this) {
	    while ( !syncDemo ) {
	      wait();
	    }
	  }
	} 
	catch (InterruptedException e){
	}
      }

      syncDemo = false;
      **/

      bufferFilled = false;

      if ( parent.northNeighbor ) {
	northBufferFilled = false;
      }
      else {
	northBufferFilled = true;
      }
      if ( parent.southNeighbor ) {
	southBufferFilled = false;
      }
      else {
	southBufferFilled = true;
      }
      if ( parent.eastNeighbor ) {
	eastBufferFilled  = false;
      }
      else {
	eastBufferFilled = true;
      }
      if ( parent.westNeighbor ) {
	westBufferFilled  = false;
      }
      else {
	westBufferFilled = true;
      }

      updateMatrix();
      parent.returnBufferColor();

      // suspend & resume point
      if ( !demoRun ) {
	try {
	  synchronized(this) {
	    while ( !demoRun ) {
	      wait();
	    }
	  }
	}
	catch (InterruptedException e){
	}
      }

    }
  }

  public synchronized void demoSuspend() {
    demoRun = false;
  }

  public synchronized void demoResume() {
    stepDemo = false;
    stepRun = true;
    demoRun = true;
    notify();
  }

  public synchronized void stepResume() {
    stepDemo = true;
    stepRun = true;
    demoRun = true;
    notify();
  }

  public synchronized void receiveData() {
    parent.receiveData();
  }

  public synchronized void allBufferFilled() {
    bufferFilled = true;
    notify();
  }

  public synchronized void syncDemonstration(int step) {
    if ( currentStep == step ) {
      syncDemo = true;
      notify();
    }
  }

  public void sendData() {
    for ( int i = 1 ; i < localRow+1 ; i++ ) {
      for ( int j = 1 ; j < localCol+1 ; j++ ) {
	parent.matrixData[j][i] = parent.newMatrixData[j][i];
      }
    }
    parent.sendData();
  }

  public void getDelay() {
    try {
      processorThread.sleep(demoSpeed);
    }
    catch(Exception e) {
    }
  }

  public void updateMatrix() {
    for ( int i = startRowIndex+1 ; i <= endRowIndex ; i++ ) {
      for ( int j = startColIndex+1 ; j <= endColIndex ; j++ ) {
	//parent.matrixData[j][i] = parent.newMatrixData[j][i];
	Double tmpD = new Double(parent.matrixData[j][i]);
	String tmpS = tmpD.toString();
	((SquarePanel)(((Vector) squares.elementAt(i-1)).elementAt(j-1))).setTopText(tmpS);
	((SquarePanel)(((Vector) squares.elementAt(i-1)).elementAt(j-1))).setBottomBackground(colorLocalNew);
	((SquarePanel)(((Vector) squares.elementAt(i-1)).elementAt(j-1))).setBottomText(" ");
      }
    }
  }

  // method       : setText
  // functionality: 
  public void setText() {
    for ( int i = 0 ; i < localRow ; i++ ) {
      for ( int j = 0 ; j < localCol ; j++ ) {
	Double tmpD = new Double(parent.matrixData[j+1][i+1]);
	String tmpS = tmpD.toString();
	//parent.parent.printLine("my ID: " + parent.myID + "   " + tmpS);
	((SquarePanel)(((Vector) squares.elementAt(i)).elementAt(j))).setTopText(tmpS);
	if ( ( !northNeighbor && i == 0 ) ||
	     ( !northNeighbor && !westNeighbor && j == 0 && i == 0 ) ||
	     ( !northNeighbor && !eastNeighbor && j == localCol - 1 && i == 0 ) ) {
	  ((SquarePanel)(((Vector) squares.elementAt(i)).elementAt(j))).setBottomText(tmpS);
	}
	else {
	  ((SquarePanel)(((Vector) squares.elementAt(i)).elementAt(j))).setBottomText(" ");
	}
      }
    }

  }

  public void initialBuffer() {
    parent.northBuffer.updateColor();
    parent.southBuffer.updateColor();
    parent.eastBuffer.updateColor();
    parent.westBuffer.updateColor();
    getDelay();
    parent.returnBufferColor();

  }


  // method       : getText
  // functionality: 
  public String getText() {
    return "aa";
  }

  protected int localRow = 0;
  protected int localCol = 0;
  protected PartitionPanel parent = null;

  protected Color colorLocalOld;
  protected Color colorLocalNew;
  protected Color colorLocalNewChanged;
  protected Color colorBoundary;
  protected Color colorBackground;

  protected Vector squares = new Vector();

  protected int processorSpeed = 100;
  protected int demoSpeed = 1000;

  public Thread processorThread;

  public boolean northBufferFilled = false;
  public boolean southBufferFilled = false;
  public boolean eastBufferFilled  = false;
  public boolean westBufferFilled  = false;

  protected boolean demoRun = true;
  protected boolean bufferFilled = false;
  protected boolean syncDemo = false;
  protected boolean stepRun = true;
  protected boolean stepDemo = false;

  protected int demoDone = 10;

  protected int startRowIndex = 0;
  protected int startColIndex = 0;

  protected int endRowIndex = 0;
  protected int endColIndex = 0;

  protected boolean northNeighbor = true;
  protected boolean southNeighbor = true;
  protected boolean eastNeighbor = true;
  protected boolean westNeighbor = true;

  protected int currentStep = -1;
}
